/* dixie.h */
/*
 * Copyright (c) 1990 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#ifdef MACOS
#include "macos.h"
#define NOUDP		1
#define NEEDINETADDR	1
#endif MACOS

#ifdef DOS
#define NOUDP	1

typedef unsigned short us;
typedef unsigned long ul;
#define ntohs(i) ((us)( (((us)i & 0xff) << 8)  + (((us)i & 0xff00) >> 8) ))
#define ntohl(i) ((ul)( (((ul)i & 0xff) << 24) + (((ul)i & 0xff00) << 8) + \
			(((ul)i & 0xff0000) >> 8) +  \
			(((ul)i & 0xff000000) >> 24) ))
#define htons(i) ((unsigned) (((unsigned)i & 0xff00) >> 8) + \
			(((unsigned) i & 0xff) << 8))

typedef unsigned long ip_addr;
typedef unsigned long u_long;
typedef unsigned char u_char;

#include <malloc.h>
#include "proto.h"
#endif /* DOS */

#define DXI_PORT 	96

#define K		1024
#define MAX_PKT_SIZE	(30*K)
#define MAX_PKT_FRAG	(K/2)

/* dixie op codes */
#define DXI_READ		0x01
#define DXI_WRITE		0x02
#define DXI_SEARCH		0x03
#define DXI_BIND		0x04
#define DXI_SEARCH1		0x05
#define DXI_ACK			0x06
#define DXI_NACK		0x07
#define DXI_SEARCH_APPROX	0x08
#define DXI_SEARCH_EXACT	0x09
#define DXI_RESEND		0x0a
#define DXI_PING		0x0b
#define DXI_SEARCH_ISUB		0x0c
#define DXI_SEARCH_FSUB		0x0d
#define DXI_SEARCH_IFSUB	0x0e
#define DXI_SEARCH_GEN		0x0f
#define DXI_LIST		0x10
#define DXI_ADD			0x11
#define DXI_REMOVE		0x12
#define DXI_MODRDN		0x13
#define DXI_COMPARE		0x14

/* dixie search scopes */
#define DXI_SRCH_BASE		0x01
#define DXI_SRCH_ONELEVEL	0x02
#define DXI_SRCH_SUBTREE	0x03

/* dixie return values */
#define DXI_SUCCESS		0x01
#define DXI_NOTFOUND		0x02
#define DXI_ERROR		0x03
#define DXI_BADCMD		0x04
#define DXI_REFUSED		0x05
#define DXI_BADDN		0x06
#define DXI_SIZELIMIT		0x07
#define DXI_NOTCONN		0x08
#define DXI_SEQNOTFOUND		0x09
#define DXI_BAD_ATTR		0x0a
#define DXI_NEX_ATTR		0x0b
#define DXI_BAD_ATTRVAL		0x0c
#define DXI_NEX_ATTRVAL		0x0d
#define DXI_NO_RDN_MOD		0x0e
#define DXI_NAME_ERROR		0x0f
#define DXI_REFER_ERROR 	0x10
#define DXI_DSA_DOWN		0x11
#define DXI_TIMELIMIT		0x12
#define DXI_TOOBIG		0x13
#define DXI_DOWN		0x14
#define DXI_EXISTS		0x15
#define DXI_COMPAREFAILED	0x16
#define DXI_BAD_FILTER		0x17

/* for compatibility ... */
#define DXI_LIMIT	0x07

#define DXI_VAL_SEP	0x01
#define DXI_ATR_SEP	0x02
#define DXI_DNS_SEP	0x03
#define DXI_ATEND_FLAG	0x04

#define DXI_H_RC		0	/* char */
#define DXI_H_OP		0	/* char */
#define DXI_H_ID		1	/* short */
#define DXI_H_TOTLEN		3	/* long */
#define DXI_H_SEQUENCE		7	/* short */
#define DXI_H_OPTIONS		9	/* char */
#define DXI_H_VERSION 		10	/* char */
#define DXI_H_SRCH_SCOPE	11	/* char */
#define DXI_H_TIMELIMIT		12	/* short */
#define DXI_H_SIZELIMIT		14	/* short */
#define DXI_H_SIZE		16	/* unused */

#define DXI_OPT_FILEATTR		1
#define DXI_OPT_DONTDEREF		2
#define DXI_OPT_DONTSEARCHALIASES	4

#define DXI_PHOTO_G3FAX_TWODIM	0x01
#define DXI_PHOTO_JPEG		0x02

#define ID	short

#if !defined(MACOS) & !defined(DOS)
extern int bprint();

extern int dxi_init();
extern int dxi_bind();
extern int dxi_udpbind();
extern int dxi_read();
extern int dxi_list();
extern int dxi_search();
extern int dxi_modify();
extern int dxi_modrdn();
extern int dxi_add();
extern int dxi_remove();
extern void dxi_perror();
extern int dxi_matches();
extern int dxi_close();
extern int dxi_compare();
extern void dxi_det61();

extern char *dxi_firstentry();
extern char *dxi_nextentry();
extern char *dxi_getdn();
extern char *dxi_lastdncomp();
extern char *dxi_firstattr();
extern char *dxi_nextattr();

extern char **dxi_getvalues();

extern void dxi_valuefree();
#endif /* MACOS DOS */

extern int dxi_debug;
extern int dxi_errno;
extern int dxi_not61;
extern char dxi_options;
extern short dxi_sizelimit;
extern short dxi_timelimit;
extern char *dxi_errlist[];
