/*
 * Copyright (c) 1990 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include "dixie.h"

#include <quipu/attrvalue.h>
#include <quipu/ds_error.h>
#include <quipu/commonarg.h>
#include <quipu/modify.h>
#include <quipu/dua.h>

#define WRITE_END(x)	{ \
		 	  dn_free(thedn); \
			  em_free(mod_arg.mea_changes); \
			  entryinfo_free(ei, 0); \
			  return(x); \
			}

extern int	debug;
extern short	timelimit, sizelimit;

static struct ds_modifyentry_arg	mod_arg = { default_common_args };
static struct DSError			mod_error;

writeop(bp, rp, dontderef)
char	*bp, *rp;
char	dontderef;
{
	DN			thedn = NULLDN;
	Attr_Sequence		str2as(), as = NULLATTR;
	Attr_Sequence		addvalues, delvalues;
	Attr_Sequence		attrs, tmpas = NULLATTR;
	AttributeType		type;
	struct entrymod		*em = NULLMOD;
	static char		buff[1024];
	int			op, i;
	int			up, err;
	EntryInfo		*ei = NULLENTRYINFO;
	struct DSError		*re;
	EntryInfo		*read_entry();

	mod_arg.mea_common.ca_servicecontrol.svc_timelimit = timelimit;
	mod_arg.mea_common.ca_servicecontrol.svc_sizelimit = sizelimit;
	if ( dontderef )
		mod_arg.mea_common.ca_servicecontrol.svc_options |=
		    SVC_OPT_DONTDEREFERENCEALIAS;
	else
		mod_arg.mea_common.ca_servicecontrol.svc_options &=
		    ~SVC_OPT_DONTDEREFERENCEALIAS;
	*rp = '\0';
	mod_arg.mea_changes = NULLMOD;

	/* get the dn whose attrs we want to modify */
	if ( *bp == '@' ) bp++;
        if ( (thedn = str2dn(bp)) == NULLDN ) {
                if ( debug ) printf("writeop: str2dn (%s) failed\n",bp);
                WRITE_END(DXI_BADDN);
        }
	mod_arg.mea_object = thedn;

	/* read attributes of the entry to be modify */
	if ( (ei = read_entry( thedn, NULLATTR, 1, 1, &up, &err, &re,dontderef ))
	    == NULLENTRYINFO ) {
		if ( debug ) printf("writeop: read_entry failed\n");
		if ( err != DXI_REFER_ERROR )
			WRITE_END( err );

		if ( chase_referral( re ) != OK )
			WRITE_END( DXI_REFER_ERROR );

		ds_error_free( &re );

		if ( (ei = read_entry( thedn, NULLATTR, 1, 1, &up, &err, 0,dontderef ))
		    == NULLENTRYINFO )
			WRITE_END( err );
	}
	ei->ent_next = NULLENTRYINFO;
	attrs = ei->ent_attr;

	bp += strlen(bp) + 1;

	/* get the attrs and values to modify */
	if ( debug ) printf("building modify values...\n");
	while ( *bp ) {
		register char	*p;
		AttributeValue	rdntype();

		strcpy(buff, bp);
		if ( debug & 8 ) printf("\t(%s)\n", buff);

		/* 
		 * find out what kind of modify this is.  The following
		 * convention is used:
		 * 	attr=value	make value the sole vlaue for attr
		 *			this is accomplished by doing a remove
		 * 			followed by an add
		 *	attr+=value	add value as another value for attr
		 *	attr-=value	remove value from attr
		 */

		/* remove attribute */
		if ( (p = index(buff, '=')) == NULL ) {
			if (debug & 8) printf("remove attribute\n");
			if ( (type = str2AttrT(buff)) == NULLAttrT ) {
				if (debug & 8) printf("bad attr\n");
				WRITE_END(DXI_BAD_ATTR);
			}

			/* check that the attr to be removed exists */
			if ( as_find_type(attrs, type) == NULLATTR ) {
				if (debug & 8)
					printf("non-existent attribute\n");
				WRITE_END(DXI_NEX_ATTR);
			}

			as = as_comp_alloc();
			as->attr_acl = NULLACL_INFO;
			as->attr_type = type;
			as->attr_link = NULLATTR;
			as->attr_value = NULLAV;

			op = EM_REMOVEATTRIBUTE;
			if (debug & 8) printf(" remove attribute\n");

		/* add attribute or add values */
		} else if ( *(p-1) == '+' ) {
			if (debug & 8) printf("add attribute or value\n");
			*(p-1) = ' ';

			if ( (as = str2as(buff)) == NULLATTR ) {
				if (debug & 8) printf("bad attr value\n");
				WRITE_END(DXI_BAD_ATTRVAL);
			}

			/* check that attr to add exists */
			if ( as_find_type(attrs, as->attr_type) == NULLATTR ) {
				op = EM_ADDATTRIBUTE;
				if (debug & 8) printf("\tadd attribute\n");
			} else {
				op = EM_ADDVALUES;
				if (debug & 8) printf("\tadd values\n");
			}

		/* remove value or attribute */
		} else if ( *(p-1) == '-' ) {
			AttributeValue	rv;

			if (debug & 8) printf("remove attribute or value\n");
			*(p-1) = ' ';

			if ( (as = str2as(buff)) == NULLATTR ) {
				if (debug & 8) printf("bad attr value\n");
				WRITE_END(DXI_BAD_ATTRVAL);
			}

			/* check that value to remove exists */
			switch ( as_find_value(attrs, as) ) {
			case -1:	/* type and last value found */
				if ( rdntype(ei->ent_dn, as->attr_type) )
					WRITE_END(DXI_NO_RDN_MOD);
				if (debug & 8) printf("\tremove attribute\n");
				op = EM_REMOVEATTRIBUTE;
				break;
			case 0:		/* type and value found */
				rv = rdntype(ei->ent_dn, as->attr_type);
				if ( rv != NULLAttrV && AttrV_cmp(rv,
				    &as->attr_value->avseq_av) == 0 )
					WRITE_END(DXI_NO_RDN_MOD);
				if (debug & 8) printf("\tremove values\n");
				op = EM_REMOVEVALUES;
				break;
			case 1:		/* type not found */
				WRITE_END(DXI_NEX_ATTR);
			case 2:		/* value not found */
				WRITE_END(DXI_NEX_ATTRVAL);
			default:	/* trouble */
				if ( debug )
					printf("bad return: as_find_value\n");
				WRITE_END(DXI_ERROR);
			}

		/* replace existing values, if any */
		} else {
			AttributeValue	rv;
			AV_Sequence		val, savevalue;
			Attr_Sequence	as_cpy();

			if (debug & 8) printf("replace values (if any)\n");

			if ( (as = str2as(buff)) == NULLATTR ) {
				if ( debug & 8 )
					printf( "str2as failed\n" );
				WRITE_END(DXI_BAD_ATTRVAL);
			}

			/* check for no values there already */
			if ( (tmpas = as_find_type(attrs, as->attr_type))
			    == NULLATTR ) {
				if (debug & 8) printf(" add attribute\n");
				op = EM_ADDATTRIBUTE;

			/* 
			 * values present so find out which get removed,
			 * added, and which can stay
			 */

			} else {
				/* 
				 * if this is the rdn type, we need to make
				 * sure the rdn value was supplied
				 */

				if ((rv = rdntype(ei->ent_dn, as->attr_type)) != NULLAttrV) {
					if (debug & 8)
						printf("RDN type -- checking that rdn is supplied\n");
					for (val = as->attr_value; val; val = val->avseq_next) {
						if (debug & 8)
							printf("\tlooking (%s)\n",val->avseq_av.av_struct);
						if (AttrV_cmp(&val->avseq_av, rv) == 0)
							break;
					}
					if (val == NULLAV) {
						if (debug & 8) printf("RDN not supplied!\n");
						WRITE_END(DXI_NO_RDN_MOD);
					}
				}

				/*
				 * ok, now we need to go through the replace list (as) and
				 * discard values already in the entry, adding values not there
				 * to the real add list (addvalues).  tmpas is a pointer
				 * to the list of values already in the entry with the type
				 * we want.
				 */

				if ( debug & 8 )
					printf("traversing replace list...\n");
				addvalues = NULLATTR;
				for ( val = as->attr_value; val; val = val->avseq_next ) {
					if (debug & 8)
						printf("\t looking (%s)\n", val->avseq_av.av_struct);

					if ( as_find_value_type( tmpas, val ) == 0 )
						continue;	/* value already there - ignore it */

					if (debug & 8)
						printf("\t\tadding...\n");

					/* value not there, so add to the addvalues list */
					if ( addvalues == NULLATTR ) {
						addvalues = as_comp_alloc();
						addvalues->attr_acl = NULLACL_INFO;
						addvalues->attr_type = AttrT_cpy(as->attr_type);
						addvalues->attr_link = NULLATTR;
						addvalues->attr_value = avs_comp_cpy(val);
					} else {
						savevalue = addvalues->attr_value;
						addvalues->attr_value = avs_comp_cpy(val);
						addvalues->attr_value->avseq_next = savevalue;
					}
					if (debug & 8)
						printf("\tadded (%s)\n",addvalues->attr_value->avseq_av.av_struct);
				}

				/*
				 * now go through the list already there (tmpas) and put
				 * any that aren't on the replace list on the to-be-deleted
				 * list (delvalues).
				 */

				if ( debug & 8 )
					printf("traversing already there list...\n");
				delvalues = NULLATTR;
				for ( val = tmpas->attr_value; val; val = val->avseq_next ) {
					if (debug & 8)
						printf("\t looking (%s)\n", val->avseq_av.av_struct);

					if ( as_find_value_type( as, val ) == 0 )
						continue;	/* value already there - ignore it */

					if (debug & 8)
						printf("\t\tadding...\n");

					/* value not there, so add to the addvalues list */
					if ( delvalues == NULLATTR ) {
						delvalues = as_comp_alloc();
						delvalues->attr_acl = NULLACL_INFO;
						delvalues->attr_type = AttrT_cpy(as->attr_type);
						delvalues->attr_link = NULLATTR;
						delvalues->attr_value = avs_comp_cpy(val);
					} else {
						savevalue = delvalues->attr_value;
						delvalues->attr_value = avs_comp_cpy(val);
						delvalues->attr_value->avseq_next = savevalue;
					}
					if (debug & 8)
						printf("\tadded (%s)\n",delvalues->attr_value->avseq_av.av_struct);
				}

				/*
				 * we add the addvalues list to the list of mods we are
				 * building here and leave the delvalues list to be added
				 * down below (via as).
				 */

				if ( addvalues != NULLATTR ) {
					if (debug & 8)
						printf("adding addvalues to modify list\n");
					if ( em == NULLMOD ) {
						em = em_alloc();
						mod_arg.mea_changes = em;
					} else {
						em->em_next = em_alloc();
						em = em->em_next;
					}
					em->em_what = addvalues;
					em->em_type = EM_ADDVALUES;
					em->em_next = NULLMOD;
				}

				as_free(as);
				as = delvalues;
				op = EM_REMOVEVALUES;
			}
		}

		bp += strlen(bp) + 1;
		if (as == NULLATTR)
			continue;

		if ( em == NULLMOD ) {
			em = em_alloc();
			mod_arg.mea_changes = em;
		} else {
			em->em_next = em_alloc();
			em = em->em_next;
		}
		em->em_what = as;
		em->em_type = op;
		em->em_next = NULLMOD;
	}

	if (debug & 16) {
		struct entrymod	*e;
		Attr_Sequence	as;
		AV_Sequence	val;

		printf("Modify changes are:\n");
		for (e = mod_arg.mea_changes; e; e = e->em_next) {
			switch (e->em_type) {
			case EM_ADDATTRIBUTE:
				printf("\tADD ATTRIBUTE\n");
				break;
			case EM_REMOVEATTRIBUTE:
				printf("\tREMOVE ATTRIBUTE\n");
				break;
			case EM_ADDVALUES:
				printf("\tADD VALUES\n");
				break;
			case EM_REMOVEVALUES:
				printf("\tREMOVE VALUES\n");
				break;
			default:
				printf("\tUNKNOWN\n");
				break;
			}

			as = e->em_what;
			printf("\t\ttype (%s)", as->attr_type->oa_ot.ot_name);
			if (e->em_type == EM_REMOVEATTRIBUTE) {
				printf("\n");
				continue;
			}
			printf(" values");
			for (val = as->attr_value; val; val = val->avseq_next)
				printf(" (%s)", val->avseq_av.av_struct);
			printf("\n");
		}
	}
	/* do the actual modify */
	if ( debug ) printf("Modify...\n");
	for ( i = 0; i < 2; i++ ) {
		if ( ds_modifyentry(&mod_arg, &mod_error) == DS_OK )
			break;
		if ( debug & 2 ) printf("ds_modifyentry failed\n");

		if ( mod_error.dse_type == DSE_REFERRAL )
			if ( chase_referral( &mod_error ) != OK )
				break;
	}
	if ( debug ) printf("Modify done ... checking errors\n");

	switch ( mod_error.dse_type ) {
	case DSE_REMOTEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &mod_error );
		}
		WRITE_END(DXI_DSA_DOWN);
	case DSE_REFERRAL:
		if ( debug & 2 ) {
			printf("\treferral error\n");
			error( &mod_error );
		}
		WRITE_END(DXI_REFER_ERROR);
	case DSE_NAMEERROR:
		if ( debug & 2 ) {
			printf("\tname error\n");
			error( &mod_error );
		}
		WRITE_END(DXI_NAME_ERROR);
	case DSE_SERVICEERROR:
		if ( debug & 2 ) {
			printf("\tservice error\n");
			error( &mod_error );
		}
		if (mod_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_TIMELIMITEXCEEDED &&
		    mod_error.dse_un.dse_un_service.DSE_sv_problem
		    != DSE_SV_ADMINLIMITEXCEEDED)
			WRITE_END(DXI_ERROR)
		break;
	case DSE_ATTRIBUTEERROR:
		if ( debug & 2 ) {
			printf("\tattribute error\n");
			error( &mod_error );
		}
		WRITE_END(DXI_EXISTS);
	case DSE_NOERROR:
		if ( debug & 2 ) printf("\tno error\n");
		break;
	default:
		if ( debug & 2 ) {
			printf("\tsome unknown error (%d)\n",
			    mod_error.dse_type);
			error( &mod_error );
		}
		WRITE_END(DXI_ERROR);
	}

	WRITE_END(DXI_SUCCESS);
}
