/*
 * Copyright (c) 1992 Regents of the University of Michigan.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of Michigan at Ann Arbor. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */

#include <stdio.h>
#include <signal.h>
#include "ud.h"

#ifndef DOS
#include <sgtty.h>
#include <sys/types.h>
#endif

char * mygetpass(prompt)
char *prompt;
{
#ifdef DOS
	static char buf[256];
	int i, c;

	printf("%s", prompt);
	i = 0;
	while ( (c = getch()) != EOF && c != '\n' && c != '\r' )
		buf[i++] = c;
	buf[i] = '\0';
	return (buf);
#else
	char i, j, k;
	struct sgttyb ttyb;
	int flags;
	static char pbuf[513];
	register char *p;
	register int c;
	FILE *fi;
	SIG_FN (*sig)();

	/*
	 *  Stolen from the getpass() routine.  Can't use the plain
	 *  getpass() for two reasons.  One is that X.500 passwords
	 *  can be really, really long - much longer than 8 chars.
	 *  The second is that we like to make this client available
	 *  out of inetd via a Merit asynch port, and we need to be
	 *  able to do telnet control codes to turn on and off line
	 *  blanking.
	 */
	if ((fi = fdopen(open("/dev/tty", 2), "r")) == NULL)
		fi = stdin;
	else
		setbuf(fi, (char *)NULL);
	sig = signal(SIGINT, SIG_IGN);
	if (fi != stdin) {
		if (ioctl(fileno(fi), TIOCGETP, (caddr_t) &ttyb) < 0)
			perror("ioctl");
	}
	flags = ttyb.sg_flags;
	ttyb.sg_flags &= ~ECHO;
	if (fi != stdin) {
		if (ioctl(fileno(fi), TIOCSETP, (caddr_t) &ttyb) < 0)
			perror("ioctl");
	}
	fprintf(stdout, "%s", prompt); fflush(stdout);

	/*  blank the line if through Merit */
	if (fi == stdin) {
		printf("%c%c%c", 255, 251, 1);
		fflush(stdout);
		(void) scanf("%c%c%c", &i, &j, &k);
		fflush(stdin);
	}
	for (p=pbuf; (c = getc(fi))!='\n' && c!=EOF;) {
		if (c == '\r')
			break;
		if (p < &pbuf[512])
			*p++ = c;
	}
	*p = '\0';
	if (*(p - 1) == '\r')
		*(p - 1) = '\0';

	/*  unblank the line if through Merit */
	if (fi == stdin) {
		printf("%c%c%c", 255, 252, 1);
		fflush(stdout);
		(void) scanf("%c%c%c", &i, &j, &k);
		fflush(stdin);
		printf("\n"); fflush(stdout);
	}
	fprintf(stdout, "\n"); fflush(stdout);
	ttyb.sg_flags = flags;
	if (fi != stdin) {
		if (ioctl(fileno(fi), TIOCSETP, (caddr_t) &ttyb) < 0)
			perror("ioctl");
	}
	(void) signal(SIGINT, sig);
	if (fi != stdin)
		(void) fclose(fi);
	else	/* XXX */
		i = getchar();
	return(pbuf);
#endif /* DOS */
}
